////////////////////////////////////////////////////
//
//	BMP.CPP: Image-class interface for BMP files.
//
//

//	This library is free software; you can redistribute it and/or
//	modify it under the terms of the GNU Library General Public
//	License as published by the Free Software Foundation; either
//	version 2 of the License, or (at your option) any later version.
//
//	This library is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//	Library General Public License for more details.
//
//	You should have received a copy of the GNU Library General Public
//	License along with this library; if not, write to the Free
//	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


#include <stdlib.h>
#include <iostream.h>
#include <fstream.h>
#include <string.h>
#include "image.h"
#include "bmp.h"

#define VERIFY(Condition,Msg) do{\
	if (!(Condition)) {\
		cerr << Msg;\
		exit(EXIT_FAILURE);\
	}\
} while(0);


BmpFile::BmpFile(ifstream &F): File(&F)
{
	File->read((char*) &header,sizeof(BmpFileHeader));
	VERIFY(!File->fail(),"Unable to read header from input file.\n")
	VERIFY(header.type == 0x4D42,"Input file not BMP format.\n");
	VERIFY(header.bitCt == 24,"Unsupported bit depth.\n");
	VERIFY(header.colorCt0 == 0,"Presence of color palette not supported.\n");
	VERIFY(header.colorCt1 == 0,"Presence of color palette not supported.\n");

	rowsLeftDat = abs(header.height);
	bytesPerRow = 3*header.width;
	bytesPadded = (bytesPerRow + 3)/4*4 - bytesPerRow;
}

void
BmpFile::getRow(pxl_t* Whither)
{
	VERIFY(rowsLeftDat-- > 0,"BmpFile::getRow: row overflow.\n");
	File->read((char*)Whither,bytesPerRow);
	VERIFY(!File->fail(),"BmpFile::getRow: read error.\n");
	File->seekg(bytesPadded,ifstream::cur);
	VERIFY(!File->fail(),"BmpFile::getRow: seek error.\n");
}

void
BmpFileFromImage(ofstream &F, Image &I)
{
	struct BmpFileHeader	Header;
	int	   BytesPerRow;
	pxl_t* 	Row = NULL;

	VERIFY(I.isFull(), "BmpFileFromImage: image depleted.\n");

	// Create and write header.
	memset(&Header,0,sizeof(BmpFileHeader));
	Header.type = 0x4D42;	// `BM'
	BytesPerRow = (3 * I.width()+3)/4*4;
	Header.size0 = 54 + BytesPerRow * abs(I.height());
	 // number of bytes in file
	Header.headerLength = 54;	// Length of this header
	Header.size1 = 40;	// Bytes from here to end of header
	Header.width = I.width();
	Header.height = I.height();
	Header.planeCt = 1;
	Header.bitCt = 24;
	Header.imageSize = BytesPerRow * abs(I.height());
	F.write((char*)&Header,sizeof(BmpFileHeader));
	VERIFY(!F.fail(),"BmpFileFromImage: error writing file header.\n");

	// Write rows of file.
	Row = (pxl_t*) new char[BytesPerRow];

	while (I.rowsLeft() > 0) {
		I.getRow(Row);
		F.write((char*)Row,BytesPerRow);
		VERIFY(!F.fail(),"BmpFileFromImage: error writing image rows.\n");
	}

	delete [] Row;
}
