////////////////////////////////////////////////////
//
//	BMP.H: Image-class interface for BMP files.
//
//

//	This library is free software; you can redistribute it and/or
//	modify it under the terms of the GNU Library General Public
//	License as published by the Free Software Foundation; either
//	version 2 of the License, or (at your option) any later version.
//
//	This library is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//	Library General Public License for more details.
//
//	You should have received a copy of the GNU Library General Public
//	License along with this library; if not, write to the Free
//	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


// requires "image.h", <iostream.h>, <fstream.h>

#pragma pack(1)
typedef struct BmpFileHeader {
	unsigned short	type;		// 0x4D42 = 'BM'
	unsigned long	size0;		// number of bytes in file
	unsigned short	reserved1;
	unsigned short	reserved2;
	unsigned long	headerLength;	// 54 = length of this header
	unsigned long	size1;		// 40 = bytes from here to end of header
	long			width;		// in pixels
	long			height;		// in pixels, negative if top-to-bottom
	unsigned short	planeCt;	// should be 1
	unsigned short	bitCt;		// should be 24
	unsigned long	compression;// should be 0 (uncompressed)
	unsigned long	imageSize;	// can set to 0
	long			xPPM;		// 0 if unknown
	long			yPPM;		// 0 if unknown
	unsigned long	colorCt0;	// should be 0
	unsigned long	colorCt1;	// should be 0
} BmpFileHeader;

class BmpFile : public Image {
private:
	struct BmpFileHeader 	header;
	ifstream				* const File;
	int						rowsLeftDat;
	int						bytesPerRow;
	int						bytesPadded;
	BmpFile(void): File(NULL) {};
		// Use BmpFile(ifstream &F) rather than this one

public:
	inline int	width(void) const { return header.width; };
	inline int	height(void) const { return header.height; };
	inline int	rowsLeft(void) const { return rowsLeftDat; };
	void 		getRow(pxl_t *Whither);
	BmpFile(ifstream &F); // Constructor
};

// Create a BMP file from an Image object.
void BmpFileFromImage(ofstream &F, Image &I);

