/*******************************************************************************
 *
 * (c) Copyright Laurence Marrie 1999
 * All Rights Reserved
 *
 ******************************************************************************/

#ifdef ASL_LINK_TIME_INSTANTIATION
#	include "asl_set.h"
#endif

template <class T>
asl_set<T>::asl_set()
	:	size_ 	( 0 )
{
}

template <class T>
asl_set<T>::asl_set( asl_set const & asl )
	:	size_	( 0 )
{
	asl_node<T> const * i = asl.end_->right();

	while ( i != asl.end_ )
	{
		insert_helper( end_, i->key() );
		i = i->right();
	}
}

template <class T>
asl_set<T>::iterator
asl_set<T>::find( T const & key ) const
{
	asl_node<T> const * node = find_helper( end_->height()-1, end_, key );

	if ( node != end_ && node->key() > key )
		node = end_;

	return iterator(node);
}

template <class T>
asl_set<T>::iterator
asl_set<T>::find( iterator const & it, T const & key ) const
{
	asl_node<T> const * node = find_with_hint_helper( it, key );

	if ( node != end_ && node->key() > key )
		node = end_;

	return iterator( node );
}

template <class T>
bool
asl_set<T>::insert( T const & key )
{
	return insert_helper( find_helper( end_->height()-1, end_, key ), key );
}

template <class T>
bool
asl_set<T>::insert( iterator const & it, T const & key )
{
	return insert_helper( find_with_hint_helper( it, key ), key );
}

template <class T>
void
asl_set<T>::erase( T const & key )
{
	asl_node<T> const * node = find_helper( end_->height()-1, end_, key );

	if ( node != end_ && !(node->key() > key) )
		erase( iterator( node ) );
}

template <class T>
void
asl_set<T>::erase( iterator it )
{
	asl_node<T> * node	= const_cast<asl_node<T>*>( it.node_ );
	asl_node<T> * right = node->right();
	asl_node<T> * left 	= node->left();
	asl_node<T> * n		= left;

	// If this node has skip pointers, swap arrays with the successor
	// or predecessor node (neither will have any). The minor optimization
	// here is to always swap with the neighbour, if any, which has another
	// neighbour also without skip pointers. This immediately restores gap
	// invariance when possible.
	if ( node->height() != 0 )
	{
		if ( left->left()->height() == 0 )
		{
			node->swapSkipArrays( *left );
		} 
		else 
		{
			node->swapSkipArrays( *right );
			n = right;
		}
	}

	// splice out
	right->setLeft( left );
	left->setRight( right );
	size_--;

	// Move to the shortest neighbour with height > 0 (if one exists)
	if ( n->height() == 0 || 
		(n->right()->height() != 0 && n->right()->height() < n->height()) )
	{
		n = n->right();
	}

	asl_node<T> * lowerLeft 	= n->left();
	asl_node<T> * lowerRight 	= n->right();

	int i = 0;	// current level
	while ( lowerLeft->height() != lowerRight->height() )
	{
		// Get nodes on both sides.
		// (Note that the 'right' and 'left' variables are being reused here,
		// though due to the alternation of the skip pointers the notion of
		// right and left may be reversed. The cases are symmetrical, so this
		// is conceptually unimportant.)
		right 		= n->next(i);
		left		= n->prev(i);

		// lower node height
		list_.lowerNode( *n );
		left->setNext( i, right );

		// Now n has same height as either lower left or lower right node.
		// If there are three nodes in a row, raise the middle one
		if ( n->height() == lowerLeft->height() )
		{
			if ( lowerLeft->prev(i-1)->height() == n->height() )
			{
				list_.raiseAfterLowering( *lowerLeft, right );
				left->setNext( i, lowerLeft );
				break;
			}
		}
		else
		{
			if ( lowerRight->next(i-1)->height() == n->height() )
			{
				list_.raiseAfterLowering( *lowerRight, right );
				left->setNext( i, lowerRight );
				break;
			}
		}

		// Move to the shorter neighbour. If this neighbour is taller, it is 
		// 1 taller (unless it is the end node). If it is not taller, then 
		// it will have height == i (in which case we will now break out of 
		// the loop)
		n = ( right->height() > left->height() ) ? left : right;
		i = n->height()-1;

		lowerRight	= n->next(i-1);
		lowerLeft	= n->prev(i-1);
	}

	// lower END node if necessary
	list_.updateEndNodeAfterLowering( *end_ );

	list_.cleanupAfterLowering();
	delete node;
}

template <class T>
void
asl_set<T>::swap( asl_set & asl )
{
	end_.swap( asl.end_ );
	list_.swap( asl.list_ );
	::swap( size_, asl.size_ );
}

template <class T>
asl_set<T> & 
asl_set<T>::operator=( asl_set const & asl )
{
	asl_set temp( asl );
	swap( temp );
	return *this;
}
