/*******************************************************************************
 *
 * (c) Copyright Laurence Marrie 1999
 * All Rights Reserved
 *
 ******************************************************************************/

#ifdef ASL_LINK_TIME_INSTANTIATION
#	include "asl_free_list.h"
#endif

#include <cstring.h>	// memcpy

template <class T>
asl_free_list<T>::asl_free_list()
	:	entries_		( 3 ),
	
		// We'll let cleanupBeforeRaising() allocate arrays_[2].next_ and 
		// arrays_[3].next_, in order to minimize initial memory requirements.
		// This is achieved by setting raiseIndex_ to 2.

		raiseIndex_		( 2 ),
		lowerIndex_		( 0 ),
		lastReleased_	( 0 )
{
	arrays_[0].size_	= 0;
	arrays_[0].next_	= 0;	// always 0

	arrays_[1].size_	= 1;
	arrays_[1].next_ 	= new asl_node<T>* [ arrays_[1].size_ ];

	// initialize remaining values to 2, 4, 8, 16, ...
	for ( int i = 2; i < FREE_LIST_SIZE; i++ )
		arrays_[i].size_ = arrays_[i-1].size_ * 2;
}

template <class T>
asl_free_list<T>::~asl_free_list()
{
	// prevent double deletion!!
	if ( raiseIndex_ != 0 )
		arrays_[raiseIndex_].next_ = 0;

	// start at 1 since the first array is always 0
	for ( int i = 1; i < entries_; i++ )
		delete [] arrays_[i].next_;
}

template <class T>
void 
asl_free_list<T>::swap( asl_free_list & afl )
{
	::swap( entries_, afl.entries_ );
	::swap( raiseIndex_, afl.raiseIndex_ );
	::swap( lowerIndex_, afl.lowerIndex_ );
	::swap( lastReleased_, afl.lastReleased_ );

	asl_free_skip_array<T> temp[FREE_LIST_SIZE];
	::memcpy( temp, arrays_, sizeof(asl_free_skip_array<T>) * FREE_LIST_SIZE );
	::memcpy( arrays_, afl.arrays_, sizeof(asl_free_skip_array<T>) * FREE_LIST_SIZE );
	::memcpy( afl.arrays_, temp, sizeof(asl_free_skip_array<T>) * FREE_LIST_SIZE );
}

template <class T>
void 
asl_free_list<T>::cleanupBeforeRaising()
{
	// The test for > 0 is actually important, to avoid a potential
	// request for 0 bytes. A peculiarity of C++ is that the call
	// "new asl_node<T>* [0]" returns a valid, not 0, pointer.
	// In our case this would result in a memory leak.
	if ( raiseIndex_ > 0 )
	{
		if ( raiseIndex_ == entries_-1 )
		{
			// Allocate a new array large enough for raising the END node
			// (placed immediately after array referenced by raiseIndex_)
			//
			arrays_[entries_].next_ 
				= new asl_node<T>* [ arrays_[entries_].size_ ];

			entries_++;		// only if allocation succeeds
		}

		// allocate array referenced by raiseIndex_
		arrays_[raiseIndex_].next_
			= new asl_node<T>* [ arrays_[raiseIndex_].size_ ];

		raiseIndex_ = 0;
	}
}

template <class T>
void
asl_free_list<T>::cleanupAfterLowering()
{
	if ( lowerIndex_-1 == raiseIndex_ )
	{
		// fill in a gap in the free list
		arrays_[raiseIndex_].next_ 	= lastReleased_;
		raiseIndex_ 				= 0;
	}
	else
		delete [] lastReleased_;

	lowerIndex_		= 0;
	lastReleased_ 	= 0;
}
