/*******************************************************************************
 *
 * (c) Copyright Laurence Marrie 1999
 * All Rights Reserved
 *
 * Purpose: A bidirectional iterator class for use with asl_set<T>.
 *
 ******************************************************************************/

#ifndef ASL_ITERATOR_H
#define ASL_ITERATOR_H

#include "asl_node.h"

template <class T>
class asl_iterator
{
public:

	asl_iterator( asl_node<T> const * node = 0 ) 	: node_( node ) {}
	asl_iterator( asl_iterator const & it )			: node_( it.node_ ) {}

	asl_iterator & operator++()		{ node_ = node_->right(); return *this; }
	asl_iterator   operator++(int)	{ asl_node<T> const * temp = node_;
										node_ = node_->right(); return temp; }

	asl_iterator & operator--()		{ node_ = node_->left(); return *this; }
	asl_iterator   operator--(int)	{ asl_node<T> const * temp = node_;
										node_ = node_->left(); return temp; }

	T const & operator*() const		{ return node_->key(); }
	T const * operator->() const 	{ return &(node_->key()); }

	bool operator==( asl_iterator const & it ) const 
									{ return node_ == it.node_; }
	bool operator!=( asl_iterator const & it ) const 
									{ return node_ != it.node_; }

	friend class asl_set<T>;

//private:
	asl_node<T> const * node_;
};

#endif
