//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//  OHCI.H
//
//  2.2.9 KERNEL
//
//  Original Authors:   Bill Alexander (www.geocities.com/SiliconValley/Haven/4824)
//                      Matt Pujol
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

#ifndef _OHCI_H_
#define _OHCI_H_

#ifndef OCTLET
//#define OCTLET unsigned long long
#define OCTLET __u64
#define POCTLET __u64 *
#endif
#ifndef QUADLET
//#define QUADLET unsigned long
#define QUADLET __u32
#define PQUADLET __u32 *
#endif
#ifndef DOUBLET
//#define DOUBLET unsigned short
#define DOUBLET __u16
#define PDOUBLET __u16 *
#endif
#ifndef BYTE
//#define BYTE unsigned char
#define BYTE __u8
#define PBYTE __u8 *
#endif
#ifndef BOOLEAN
#define BOOLEAN unsigned int
#endif
#ifndef TRUE
#define TRUE 1
#endif
#ifndef FALSE
#define FALSE 0
#endif
#ifndef BoolToStr
#define BoolToStr(b) b==TRUE?"TRUE":"FALSE"
#endif
#ifndef QUADLET_WRITE
#define QUADLET_WRITE(pmmap,offset,value) *((QUADLET *)(((BYTE *)pmmap)+offset))=value
#endif
#ifndef QUADLET_READ
#define QUADLET_READ(pmmap,offset) *((QUADLET *)(((BYTE *)pmmap)+offset))
#endif
#ifndef REG_WRITE
#define REG_WRITE(offset,value) DBOUT2(DBG_WRITE_REGS,"%08X->reg %08X\n",value,offset);QUADLET_WRITE(gOhciRegMemMap,offset,value)
#endif
#ifndef REG_READ
#ifdef DEBUG
static QUADLET g_dbg_tmp_rd_reg_val;
#define REG_READ(offset) g_dbg_tmp_rd_reg_val=QUADLET_READ(gOhciRegMemMap,offset);DBOUT2(DBG_READ_REGS,"%08X<-reg %08X\n",g_dbg_tmp_rd_reg_val,offset) 
#else
#define REG_READ(offset) QUADLET_READ(gOhciRegMemMap,offset)
#endif
#endif
#ifndef DMA_WRITE
#define DMA_WRITE(offset,value) QUADLET_WRITE(gOhciDmaMemMap,offset,value)
#endif
#ifndef DMA_READ
#define DMA_READ(offset) QUADLET_READ(gOhciDmaMemMap,offset)
#endif
#ifndef offsetof
#define offsetof(ttype,field) (unsigned)&(((ttype *)0)->field)
#endif
#ifndef MyIsPrint
#define MyIsPrint(c) (c>=' ' && c<='~')
#endif

// Major Device Number
#define OHCI_MAJOR_DEV_NUM	254

//-------------------------------------------
//  IOCTL COMMANDS
//-------------------------------------------
#define OHCIQUADSEND                0x51
#define OHCIINFO                    0x52
#define OHCISEND                    0x53
#define OHCIGETREGPAGEALIGNOFFSET   0x54
#define OHCIGETDMAPAGEPHYSADDR      0x55

#define Version 0x000
typedef union
{
    QUADLET reg;
    struct
    {
        __u32 revision :8;
        __u32 Rsvd :16;
        __u32 version: 8;
        __u32 GUID_ROM: 1;
        __u32 Rsvd1 :7;
    }fld;
}Version_REG;

#define GUID_ROM    0x004
typedef union
{
    QUADLET reg;
    struct
    {
        __u32 Rsvd :16;
        __u32 rdData :8;
        __u32 Rsvd1 :1;
        __u32 rdStart :1;
        __u32 Rsvd2 :5;
        __u32 addrReset :1;
    }fld;
}GUID_ROM_REG;


#define ATRetries   0x008
typedef union
{
    QUADLET reg;
    struct
    {
        __u32 maxATReqRetries :4;
        __u32 maxATRespRetries :4;
        __u32 maxPhysRespRetries :4;
        __u32 Rsvd :4;
        __u32 cycleLimit :13;
        __u32 secondLimit :3;
    }fld;
}ATRetries_REG;

#define     CSRData    0x00C
typedef union
{
    QUADLET reg;
    __u32 csrData :32;
}CSRData_REG;

#define CSRCompareData  0x010
typedef union
{
    QUADLET reg;
    struct
    {
        __u32 csrCompare :32;
    }fld;
}CSRCompareData_REG;

#define CSRControl 0x014
typedef union
{
    QUADLET reg;
    struct
    {
        __u32 rsvd0 :16;
        __u32 rdData :8;
        __u32 rsvd1 :1;
        __u32 rdStart :1;
        __u32 rsvd2 :5;
        __u32 addrReset :1;
    }fld;
}CSRControl_REG;

#define ConfigROMheader 0x018
#define ConfigROMhdr    0x018
typedef union
{
    QUADLET reg;
    struct
    {
        __u32 rom_crc_value :16;
        __u32 crc_length :8;
        __u32 info_length :8;
    }fld;
}ConfigROMheader_REG,
 ConfigRomhdr_REG;

#define   BusID          0x01C
typedef union
{
    QUADLET reg;
    struct
    {
        __u32 link_spd :3;
        __u32 rsvd0 :3;
        __u32 g :2;
        __u32 rsvd1 :4;
        __u32 max_rec :4;
        __u32 cyc_clk_acc :8;
        __u32 rsvd2 :3;
        __u32 pmc :1;
        __u32 bmc :1;
        __u32 isc :1;
        __u32 cmc :1;
        __u32 irmc :1;
    }fld;
}BusID_REG;

#define BusOptions  0x020
typedef union
{
    QUADLET reg;
    struct
    {
        __u32 link_spd :3;
        __u32 Rsvd :3;
        __u32 g :1;
        __u32 Rsvd1 :4;
        __u32 max_rec :4;
        __u32 cyc_clk_acc :8;
        __u32 Rsvd2 :3;
        __u32 pmc :1;
        __u32 bmc :1;
        __u32 isc :1;
        __u32 cmc :1;
        __u32 irmc :1;
    }fld;
}BusOptions_REG;

#define   GUIDHi         0x024
typedef union
{
    QUADLET reg;
    struct
    {
        __u32 chip_ID_hi :8;
        __u32 node_vendor_ID :24;
    }fld;
}GUIDHi_REG;

#define GUIDLo  0x028
#define GlobalUniqueIDLo    0x028
typedef union
{
    QUADLET reg;
    struct
    {
        __u32 chip_ID_Lo :32;
    }fld;
}GUIDLo_REG,
 GlobalUniqueIDLo_REG;

#define ConfigROMmap    0x034
typedef union
{
    QUADLET reg;
    struct
    {
        __u32 Rsvd :10;
        __u32 configROMaddr :7;
    }fld;
}ConfigROMmap_REG;


#define PostedWriteAddressLo    0x038
typedef union
{
    QUADLET reg;
    struct
    {
        __u32 offsetLo :32;
    }fld;
}PostedWriteAddressLo_REG;

#define   PostedWriteAddressHi         0x03C
typedef union
{
    QUADLET reg;
    struct
    {
        __u32 offsetHi :16;
        __u32 sourceiD :16;
    }fld;
}PostedWriteAddressHi_REG;

#define   VendorID         0x040
typedef union
{
    QUADLET reg;
    struct
    {
        __u32 vendorCompanyID :24;
        __u32 vendorUnique :8;
    }fld;
}VendorID_REG;

#define   HCControlSet         0x050
#define   HCControlClear         0x054
typedef union
{
    QUADLET reg;
    struct
    {
        __u32 Rsvd :16;
        __u32 softReset :1;
        __u32 linkEnable :1;
        __u32 postedWriteEnable :1;
        __u32 LPS :1;
        __u32 Rsvd1 :2;
        __u32 aPhyEnhanceEnable :1;
        __u32 programPhyEnable :1;
        __u32 Rsvd2 :6;
        __u32 noByteSwapData :1;
        __u32 Rsvd3 :1;
    }fld;
}HCControlSet_REG, HCControlClear_REG;

#define   SelfIDBuffer         0x064
typedef union
{
    QUADLET reg;
    struct
    {
        __u32 Rsvd :11;
        __u32 selfIDBufferPtr :21;
    }fld;
}SelfIDBuffer_REG;

#define   SelfIDCount         0x068
typedef union
{
    QUADLET reg;
    struct
    {
        __u32 Rsvd :2;
        __u32 selfIDSize :11;
        __u32 Rsvd1 :3;
        __u32 selfIDGeneration :8;
        __u32 Rsvd2 :7;
        __u32 selfIDError :1;
    }fld;
}SelfIDCount_REG;

#define IRMultiChanMaskHiSet    0x070
#define IRMultiChanMaskHiClear  0x074
typedef union
{
    QUADLET reg;
    struct
    {
        __u32 isoChannel32 :1;
        __u32 isoChannel33 :1;
        __u32 isoChannel34 :1;
        __u32 isoChannel35 :1;
        __u32 isoChannel36 :1;
        __u32 isoChannel37 :1;
        __u32 isoChannel38 :1;
        __u32 isoChannel39 :1;
        __u32 isoChannel40 :1;
        __u32 isoChannel41 :1;
        __u32 isoChannel42 :1;
        __u32 isoChannel43 :1;
        __u32 isoChannel44 :1;
        __u32 isoChannel45 :1;
        __u32 isoChannel46 :1;
        __u32 isoChannel47 :1;
        __u32 isoChannel48 :1;
        __u32 isoChannel49 :1;
        __u32 isoChannel50 :1;
        __u32 isoChannel51 :1;
        __u32 isoChannel52 :1;
        __u32 isoChannel53 :1;
        __u32 isoChannel54 :1;
        __u32 isoChannel55 :1;
        __u32 isoChannel56 :1;
        __u32 isoChannel57 :1;
        __u32 isoChannel58 :1;
        __u32 isoChannel59 :1;
        __u32 isoChannel60 :1;
        __u32 isoChannel61 :1;
        __u32 isoChannel62 :1;
        __u32 isoChannel63 :1;
    }fld;
}IRMultiChanMaskHiSet_REG,
 IRMultiChanMaskHiClear_REG;


#define IRMultiChanMaskLoSet    0x078
#define IRMultiChanMaskLoClear  0x07C
typedef union
{
    QUADLET reg;
    struct
    {
        __u32 isoChannel0 :1;
        __u32 isoChannel1 :1;
        __u32 isoChannel2 :1;
        __u32 isoChannel3 :1;
        __u32 isoChannel4 :1;
        __u32 isoChannel5 :1;
        __u32 isoChannel6 :1;
        __u32 isoChannel7 :1;
        __u32 isoChannel8 :1;
        __u32 isoChannel9 :1;
        __u32 isoChannel10 :1;
        __u32 isoChannel11 :1;
        __u32 isoChannel12 :1;
        __u32 isoChannel13 :1;
        __u32 isoChannel14 :1;
        __u32 isoChannel15 :1;
        __u32 isoChannel16 :1;
        __u32 isoChannel17 :1;
        __u32 isoChannel18 :1;
        __u32 isoChannel19 :1;
        __u32 isoChannel20 :1;
        __u32 isoChannel21 :1;
        __u32 isoChannel22 :1;
        __u32 isoChannel23 :1;
        __u32 isoChannel24 :1;
        __u32 isoChannel25 :1;
        __u32 isoChannel26 :1;
        __u32 isoChannel27 :1;
        __u32 isoChannel28 :1;
        __u32 isoChannel29 :1;
        __u32 isoChannel30 :1;
        __u32 isoChannel31 :1;
    }fld;
}IRMultiChanMaskLo_REG,
 IRMultiChanMaskLoSet_REG,
 IRMultiChanMaskLoClear_REG;

#define IntEventSet     0x080
#define IntEvent        0x080
#define IntEventClear   0x084
#define IntMask         0x088
#define IntMaskSet      0x088
#define IntMaskClear    0x08C

#define reqTxComplete_BIT       0x00000001
#define respTxComplete_BIT      0x00000002
#define ARRQ_BIT                0x00000004
#define ARRS_BIT                0x00000008
#define RQPkt_BIT               0x00000010
#define RSPkt_BIT               0x00000020
#define isochTx_BIT             0x00000040
#define isochRx_BIT             0x00000080
#define postedWriteErr_BIT      0x00000100
#define lockRespErr_BIT         0x00000200
#define selfIDComplete_BIT      0x00010000
#define busReset_BIT            0x00020000
#define phy_BIT                 0x00080000
#define cycleSynch_BIT          0x00100000
#define cycle64Seconds_BIT      0x00200000
#define cycleLost_BIT           0x00400000
#define cycleInconsistent_BIT   0x00800000
#define unrecoverableError_BIT  0x01000000
#define cycleTooLong_BIT        0x02000000
#define phyRegRcvd_BIT          0x04000000
#define vendorSpecific_BIT      0x40000000

typedef union
{
    QUADLET reg;
    struct
    {
        __u32 reqTxComplete     :1;
        __u32 respTxComplete        :1;
        __u32 ARRQ              :1;
        __u32 ARRS              :1;
        __u32 RQPkt             :1;
        __u32 RSPkt             :1;
        __u32 isochTx           :1;
        __u32 isochRx           :1;
        __u32 postedWriteErr        :1;
        __u32 lockRespErr       :1;
        __u32 Rsvd1             :6;
        __u32 selfIDComplete        :1;
        __u32 busReset          :1;
        __u32 Rsvd2             :1;
        __u32 phy               :1;
        __u32 cycleSynch            :1;
        __u32 cycle64Seconds        :1;
        __u32 cycleLost         :1;
        __u32 cycleInconsistent :1;
        __u32 unrecoverableError :1;
        __u32 cycleTooLong      :1;
        __u32 phyRegRcvd            :1;
        __u32 Rsvd3             :3;
        __u32 vendorSpecific        :1;
        __u32 Rsvd4             :1;
    }fld;
}IntEvent_REG, 
 IntEventSet_REG, 
 IntEventClear_REG,
 IntMask_REG, 
 IntMaskSet_REG, 
 IntMaskClear_REG;


#define isoXmitIntEvent 0x090
#define isoXmitIntEventSet  0x090
#define isoXmitIntEventClear    0x094
#define isoXmitIntMask  0x098
#define isoXmitIntMaskSet   0x098
#define isoXmitIntMaskClear 0x09C
typedef union
{
    QUADLET reg;
    struct
    {
        __u32 isoXmit0 :1;
        __u32 isoXmit1 :1;
        __u32 isoXmit2 :1;
        __u32 isoXmit3 :1;
        __u32 isoXmit4 :1;
        __u32 isoXmit5 :1;
        __u32 isoXmit6 :1;
        __u32 isoXmit7 :1;
        __u32 isoXmit8 :1;
        __u32 isoXmit9 :1;
        __u32 isoXmit10 :1;
        __u32 isoXmit11 :1;
        __u32 isoXmit12 :1;
        __u32 isoXmit13 :1;
        __u32 isoXmit14 :1;
        __u32 isoXmit15 :1;
        __u32 isoXmit16 :1;
        __u32 isoXmit17 :1;
        __u32 isoXmit18 :1;
        __u32 isoXmit19 :1;
        __u32 isoXmit20 :1;
        __u32 isoXmit21 :1;
        __u32 isoXmit22 :1;
        __u32 isoXmit23 :1;
        __u32 isoXmit24 :1;
        __u32 isoXmit25 :1;
        __u32 isoXmit26 :1;
        __u32 isoXmit27 :1;
        __u32 isoXmit28 :1;
        __u32 isoXmit29 :1;
        __u32 isoXmit30 :1;
        __u32 isoXmit31 :1;
    }fld;
}isoXmitIntEvent_REG,
 isoXmitIntEventSet_REG,
 isoXmitIntEventClear_REG,
 isoXmitIntMask_REG,
 isoXmitIntMaskSet_REG,
 isoXmitIntMaskClear_REG;

#define isoRecvIntEvent 0x090
#define isoRecvIntEventSet  0x090
#define isoRecvIntEventClear    0x094
#define isoRecvIntMask  0x098
#define isoRecvIntMaskSet   0x098
#define isoRecvIntMaskClear 0x09C
typedef union
{
    QUADLET reg;
    struct
    {
        __u32 isoRecv0 :1;
        __u32 isoRecv1 :1;
        __u32 isoRecv2 :1;
        __u32 isoRecv3 :1;
        __u32 isoRecv4 :1;
        __u32 isoRecv5 :1;
        __u32 isoRecv6 :1;
        __u32 isoRecv7 :1;
        __u32 isoRecv8 :1;
        __u32 isoRecv9 :1;
        __u32 isoRecv10 :1;
        __u32 isoRecv11 :1;
        __u32 isoRecv12 :1;
        __u32 isoRecv13 :1;
        __u32 isoRecv14 :1;
        __u32 isoRecv15 :1;
        __u32 isoRecv16 :1;
        __u32 isoRecv17 :1;
        __u32 isoRecv18 :1;
        __u32 isoRecv19 :1;
        __u32 isoRecv20 :1;
        __u32 isoRecv21 :1;
        __u32 isoRecv22 :1;
        __u32 isoRecv23 :1;
        __u32 isoRecv24 :1;
        __u32 isoRecv25 :1;
        __u32 isoRecv26 :1;
        __u32 isoRecv27 :1;
        __u32 isoRecv28 :1;
        __u32 isoRecv29 :1;
        __u32 isoRecv30 :1;
        __u32 isoRecv31 :1;
    }fld;
}isoRecvIntEvent_REG,
 isoRecvIntEventSet_REG,
 isoRecvIntEventClear_REG,
 isoRecvIntMask_REG,
 isoRecvIntMaskSet_REG,
 isoRecvIntMaskClear_REG;

#define FairnessControl 0x0DC
typedef union
{
    QUADLET reg;
    struct
    {
        __u32 pri_req :8;
        __u32 Rsvd :24;
    }fld;
}FairnessControl_REG;

#define LinkControl 0x0E0
#define LinkControlSet  0x0E0
#define LinkControlClear    0x0E4
typedef union
{
    QUADLET reg;
    struct
    {
        __u32 Rsvd :9;
        __u32 rcvSelfID :1;
        __u32 rcvPhyPkt :1;
        __u32 Rsvd1 :9;
        __u32 cycleTimerEnable :1;
        __u32 cycleMaster :1;
        __u32 cycleSource :1;
        __u32 Rsvd2 :9;
    }fld;
}LinkControl_REG, LinkControlSet_REG, LinkControlClear_REG;


#define NodeID  0x0E8
typedef union
{
    QUADLET reg;
    struct
    {
        __u32 nodeNumber :6;
        __u32 busNumber :10;
        __u32 Rsvd :11;
        __u32 CPS :1;
        __u32 Rsvd1 :2;
        __u32 root :1;
        __u32 idValid :1;
    }fld;
}NodeID_REG;

#define PHYControl  0x0EC
typedef union
{
    QUADLET reg;
    struct
    {
        __u32 wrData :8;
        __u32 regAddr :4;
        __u32 Rsvd :2;
        __u32 wrReg :1;
        __u32 rdReg :1;
        __u32 rdData :8;
        __u32 rdAddr :4;
        __u32 Rsvd1 :3;
        __u32 rdDone :1;
    }fld;
}PHYControl_REG;

#define IsochronousCycleTimer   0x0F0
typedef union
{
    QUADLET reg;
    struct
    {
        __u32 cycleOffset :12;
        __u32 cycleCount :13;
        __u32 cycleSeconds :7;
    }fld;
}IsochronousCycleTimer_REG;

#define AsynchronousRequestFilterHi 0x100
#define AsynchronousRequestFilterHiSet  0x100
#define AsynchronousRequestFilterHiClear    0x104
typedef union
{
    QUADLET reg;
    struct
    {
        __u32 asynReqResource32 :1;
        __u32 asynReqResource33 :1;
        __u32 asynReqResource34 :1;
        __u32 asynReqResource35 :1;
        __u32 asynReqResource36 :1;
        __u32 asynReqResource37 :1;
        __u32 asynReqResource38 :1;
        __u32 asynReqResource39 :1;
        __u32 asynReqResource40 :1;
        __u32 asynReqResource41 :1;
        __u32 asynReqResource42 :1;
        __u32 asynReqResource43 :1;
        __u32 asynReqResource44 :1;
        __u32 asynReqResource45 :1;
        __u32 asynReqResource46 :1;
        __u32 asynReqResource47 :1;
        __u32 asynReqResource48 :1;
        __u32 asynReqResource49 :1;
        __u32 asynReqResource50 :1;
        __u32 asynReqResource51 :1;
        __u32 asynReqResource52 :1;
        __u32 asynReqResource53 :1;
        __u32 asynReqResource54 :1;
        __u32 asynReqResource55 :1;
        __u32 asynReqResource56 :1;
        __u32 asynReqResource57 :1;
        __u32 asynReqResource58 :1;
        __u32 asynReqResource59 :1;
        __u32 asynReqResource60 :1;
        __u32 asynReqResource61 :1;
        __u32 asynReqResource62 :1;
        __u32 asynReqResourceAll :1;
    }fld;
}AsynchronousRequestFilterHi_REG,
 AsynchronousRequestFilterHiSet_REG,
 AsynchronousRequestFilterHiClear_REG;

#define AsynchronousRequestFilterLo 0x108
#define AsynchronousRequestFilterLoSet  0x108
#define AsynchronousRequestFilterLoClear    0x10C
typedef union
{
    QUADLET reg;
    struct
    {
        __u32 asynReqResource0 :1;
        __u32 asynReqResource1 :1;
        __u32 asynReqResource2 :1;
        __u32 asynReqResource3 :1;
        __u32 asynReqResource4 :1;
        __u32 asynReqResource5 :1;
        __u32 asynReqResource6 :1;
        __u32 asynReqResource7 :1;
        __u32 asynReqResource8 :1;
        __u32 asynReqResource9 :1;
        __u32 asynReqResource10 :1;
        __u32 asynReqResource11 :1;
        __u32 asynReqResource12 :1;
        __u32 asynReqResource13 :1;
        __u32 asynReqResource14 :1;
        __u32 asynReqResource15 :1;
        __u32 asynReqResource16 :1;
        __u32 asynReqResource17 :1;
        __u32 asynReqResource18 :1;
        __u32 asynReqResource19 :1;
        __u32 asynReqResource20 :1;
        __u32 asynReqResource21 :1;
        __u32 asynReqResource22 :1;
        __u32 asynReqResource23 :1;
        __u32 asynReqResource24 :1;
        __u32 asynReqResource25 :1;
        __u32 asynReqResource26 :1;
        __u32 asynReqResource27 :1;
        __u32 asynReqResource28 :1;
        __u32 asynReqResource29 :1;
        __u32 asynReqResource30 :1;
        __u32 asynReqResource31 :1;
    }fld;
}AsynchronousRequestFilterLo_REG,
 AsynchronousRequestFilterLoSet_REG,
 AsynchronousRequestFilterLoClear_REG;

#define PhysicalRequestFilterHi 0x110
#define PhysicalRequestFilterHiSet  0x110
#define PhysicalRequestFilterHiClear    0x114
typedef union
{
    QUADLET reg;
    struct
    {
        __u32 physReqResource32 :1;
        __u32 physReqResource33 :1;
        __u32 physReqResource34 :1;
        __u32 physReqResource35 :1;
        __u32 physReqResource36 :1;
        __u32 physReqResource37 :1;
        __u32 physReqResource38 :1;
        __u32 physReqResource39 :1;
        __u32 physReqResource40 :1;
        __u32 physReqResource41 :1;
        __u32 physReqResource42 :1;
        __u32 physReqResource43 :1;
        __u32 physReqResource44 :1;
        __u32 physReqResource45 :1;
        __u32 physReqResource46 :1;
        __u32 physReqResource47 :1;
        __u32 physReqResource48 :1;
        __u32 physReqResource49 :1;
        __u32 physReqResource50 :1;
        __u32 physReqResource51 :1;
        __u32 physReqResource52 :1;
        __u32 physReqResource53 :1;
        __u32 physReqResource54 :1;
        __u32 physReqResource55 :1;
        __u32 physReqResource56 :1;
        __u32 physReqResource57 :1;
        __u32 physReqResource58 :1;
        __u32 physReqResource59 :1;
        __u32 physReqResource60 :1;
        __u32 physReqResource61 :1;
        __u32 physReqResource62 :1;
        __u32 physReqResourceAllBuses :1;
    }fld;
}PhysicalRequestFilterHi_REG,
 PhysicalRequestFilterHiSet_REG,
 PhysicalRequestFilterHiClear_REG;

#define PhysicalRequestFilterLo 0x118
#define PhysicalRequestFilterLoSet  0x118
#define PhysicalRequestFilterLoClear    0x11C
typedef union
{
    QUADLET reg;
    struct
    {
        __u32 physReqResource0 :1;
        __u32 physReqResource1 :1;
        __u32 physReqResource2 :1;
        __u32 physReqResource3 :1;
        __u32 physReqResource4 :1;
        __u32 physReqResource5 :1;
        __u32 physReqResource6 :1;
        __u32 physReqResource7 :1;
        __u32 physReqResource8 :1;
        __u32 physReqResource9 :1;
        __u32 physReqResource10 :1;
        __u32 physReqResource11 :1;
        __u32 physReqResource12 :1;
        __u32 physReqResource13 :1;
        __u32 physReqResource14 :1;
        __u32 physReqResource15 :1;
        __u32 physReqResource16 :1;
        __u32 physReqResource17 :1;
        __u32 physReqResource18 :1;
        __u32 physReqResource19 :1;
        __u32 physReqResource20 :1;
        __u32 physReqResource21 :1;
        __u32 physReqResource22 :1;
        __u32 physReqResource23 :1;
        __u32 physReqResource24 :1;
        __u32 physReqResource25 :1;
        __u32 physReqResource26 :1;
        __u32 physReqResource27 :1;
        __u32 physReqResource28 :1;
        __u32 physReqResource29 :1;
        __u32 physReqResource30 :1;
        __u32 physReqResource31 :1;
    }fld;
}PhysicalRequestFilterLo_REG,
 PhysicalRequestFilterLoSet_REG,
 PhysicalRequestFilterLoClear_REG;

#define PhysicalUpperBound  0x120
typedef union
{
    QUADLET reg;
    struct
    {
        __u32 physUpperBoundOffset :32;
    }fld;
}PhysicalUpperBound_REG;

#define AsyncRequestTransmitContextControlSet       0x180
#define AsyncRequestTransmitContextControlClear     0x184
#define AsyncResponseTransmitContextControlSet      0x1A0
#define AsyncResponseTransmitContextControlClear    0x1A4
#define AsyncRequestReceiveContextControlSet        0x1C0
#define AsyncRequestReceiveContextControlClear      0x1C4
#define AsyncResponseReceiveContextControlSet       0x1E0
#define AsyncResponseReceiveContextControlClear     0x1E4
typedef union
{
    QUADLET reg;
    struct
    {
        __u32 event_code :5;
        __u32 Rsvd :3;
        __u32 Rsvd1 :2;
        __u32 active :1;
        __u32 dead :1;
        __u32 wake :1;
        __u32 Rsvd2 :2;
        __u32 run :1;
        __u32 Rsvd3 :16;
    }fld;
}AsyncRequestTransmitContextControl_REG,
 AsyncRequestTransmitContextControlSet_REG,
 AsyncRequestTransmitContextControlClear_REG,
 AsyncResponseTransmitContextControl_REG,
 AsyncResponseTransmitContextControlSet_REG,
 AsyncResponseTransmitContextControlClear_REG,
 AsyncRequestReceiveContextControl_REG,
 AsyncRequestReceiveContextControlSet_REG,
 AsyncRequestReceiveContextControlClear_REG,
 AsyncResponseReceiveContextControl_REG,
 AsyncResponseReceiveContextControlSet_REG,
 AsyncResponseReceiveContextControlClear_REG;


#define AsyncRequestTransmitCommandPtr  0x18C
#define AsyncResponseTransmitCommandPtr 0x1AC
#define AsyncRequestReceiveCommandPtr   0x1CC
#define AsyncResponseReceiveCommandPtr  0x1EC
#define IsochTransmitNCommandPtrBase    0x20C
#define IsochTransmitCommandPtr(n)      ((n*16)+IsochTransmitNCommandPtrBase)
typedef union
{
    QUADLET reg;
    struct
    {
        __u32 Z :4;
        __u32 descriptorAddress :28;
    }fld;
}AsyncRequestTransmitCommandPtr_REG,
 AsyncResponseTransmitCommandPtr_REG,
 AsyncRequestReceiveCommandPtr_REG,
 AsyncResponseReceiveCommandPtr_REG,
 IsochTransmitCommandPtr_REG;


#define IsochTransmitNContextControlSetBase     0x200
#define IsochTransmitNContextControlClearBase   0x204
#define IsochTransmitContextControlSet(n)       ((n*16)+IsochTransmitNContextControlSetBase)
#define IsochTransmitContextControlClear(n)     ((n*16)+IsochTransmitNContextControlClearBase)
typedef union
{
    QUADLET reg;
    struct
    {
        __u32 event_code :5;
        __u32 Rsvd :3;
        __u32 Rsvd1 :2;
        __u32 active :1;
        __u32 dead :1;
        __u32 wake :1;
        __u32 Rsvd2 :2;
        __u32 run :1;
        __u32 cycleMatch :15;
        __u32 cycleMatchEnabled :1;
    }fld;
}IsochTransmitContextControl_REG,
 IsochTransmitContextControlSet_REG,
 IsochTransmitContextControlClear_REG;

// PHY Registers -- accessed via PHYControl
typedef union
{
    BYTE reg;
    struct
    {
        BYTE PS             :1;
        BYTE R              :1;
        BYTE Physical_ID    :6;
    }fld;
} PHYRegister0_REG;
#define PHYRegister0        0x0
typedef union
{
    BYTE reg;
    struct
    {
        BYTE Gap_count      :6;
        BYTE IBR            :1;
        BYTE RHB            :1;
    }fld;
} PHYRegister1_REG;
#define PHYRegister1        0x1
typedef union
{
    BYTE reg;
    struct
    {
        BYTE NumPorts       :5;
        BYTE E              :1;
        BYTE SPD            :2;
    }fld;
} PHYRegister2_REG;
#define PHYRegister2        0x2
typedef union
{
    BYTE reg;
    struct
    {
        BYTE Rsvd           :2;
        BYTE Con            :1;
        BYTE Ch             :1;
        BYTE BStat          :2;
        BYTE AStat          :2;
    }fld;
} PHYRegister3_REG, 
  PHYStatusRegister_REG;
#define PHYRegister3            0x3
#define PHYNStatusRegisterBase  0x3
#define PHYStatusRegister(n)    n+PHYNStatusRegisterBase


// Self ID Receive structure
typedef union
{
    QUADLET reg[2];
    struct
    {
        QUADLET m           :1;
        QUADLET i           :1;
        QUADLET p2          :2;
        QUADLET p1          :2;
        QUADLET p0          :2;
        QUADLET pwr         :3;
        QUADLET c           :1;
        QUADLET del         :2;
        QUADLET sp          :2;
        QUADLET gap_cnt     :6;
        QUADLET L           :1;
        QUADLET Zero        :1;     // Should always be 0
        QUADLET phy_ID      :6;
        QUADLET OneZero     :2;     // Should always be 10
        QUADLET LogicalInverse;     // Inverse of first quadlet
    } Packet0;
    struct
    {
        QUADLET m           :1;
        QUADLET r           :1;
        QUADLET p10         :2;
        QUADLET p9          :2;
        QUADLET p8          :2;
        QUADLET p7          :2;
        QUADLET p6          :2;
        QUADLET p5          :2;
        QUADLET p4          :2;
        QUADLET p3          :2;
        QUADLET rsv         :2;
        QUADLET n           :3;     // Should be 0 for Packet1
        QUADLET One         :1;     // Should always be 1
        QUADLET phy_ID      :6;
        QUADLET OneZero     :2;     // Should always be 10
        QUADLET LogicalInverse;     // Inverse of first quadlet
    } Packet1;
    struct
    {
        QUADLET m           :1;
        QUADLET r           :1;
        QUADLET p18         :2;
        QUADLET p17         :2;
        QUADLET p16         :2;
        QUADLET p15         :2;
        QUADLET p14         :2;
        QUADLET p13         :2;
        QUADLET p12         :2;
        QUADLET p11         :2;
        QUADLET rsv         :2;
        QUADLET n           :3;     // Should be 1 for Packet2
        QUADLET One         :1;     // Should always be 1
        QUADLET phy_ID      :6;
        QUADLET OneZero     :2;     // Should always be 10
        QUADLET LogicalInverse;     // Inverse of first quadlet
    } Packet2;
    struct
    {
        QUADLET m           :1;
        QUADLET r           :1;
        QUADLET p26         :2;
        QUADLET p25         :2;
        QUADLET p24         :2;
        QUADLET p23         :2;
        QUADLET p22         :2;
        QUADLET p21         :2;
        QUADLET p20         :2;
        QUADLET p19         :2;
        QUADLET rsv         :2;
        QUADLET n           :3;     // Should be 2 for Packet3
        QUADLET One         :1;     // Should always be 1
        QUADLET phy_ID      :6;
        QUADLET OneZero     :2;     // Should always be 10
        QUADLET LogicalInverse;     // Inverse of first quadlet
    } Packet3;
    struct
    {
        QUADLET m           :1;
        QUADLET r           :1;
        QUADLET ph          :2;
        QUADLET pg          :2;
        QUADLET pf          :2;
        QUADLET pe          :2;
        QUADLET pd          :2;
        QUADLET pc          :2;
        QUADLET pb          :2;
        QUADLET pa          :2;
        QUADLET rsv         :2;
        QUADLET n           :3;     // Should be N-1 for PacketN
        QUADLET One         :1;     // Should always be 1
        QUADLET phy_ID      :6;
        QUADLET OneZero     :2;     // Should always be 10
        QUADLET LogicalInverse;     // Inverse of first quadlet
    } PacketN;
} SelfIDPacket;
#define MAX_OHCI_SELF_ID_PKTS   63*4
typedef struct SelfIDReceiveStruct
{
    QUADLET         timeStamp           :16;
    QUADLET         selfIDGeneration    :8;
    QUADLET         Rsvd                :8;
    SelfIDPacket        Packet[MAX_OHCI_SELF_ID_PKTS];  // possible 63 nodes * 4 packets/node 
} SelfIDReceive;

// DB DMA Descriptors and Definitions

#define CMD_OUTPUT_MORE                 0
#define CMD_OUTPUT_LAST                 1
#define CMD_INPUT_MORE                  2
#define CMD_STORE_VALUE                 8         

#define KEY_ADDRESS                     0
#define KEY_1                           1
#define KEY_IMMEDIATE                   2
#define KEY_3                           3
#define KEY_4                           4
#define KEY_5                           5
#define KEY_SYSTEM_MEMORY               6
#define KEY_7                           7

#define BRANCH_NEVER                    0
#define BRANCH_ON_1                     1
#define BRANCH_ON_2                     2
#define BRANCH_ALWAYS                   3

#define INTERRUPT_NEVER                 0
#define INTERRUPT_NO_ACK                1
#define INTERRUPT_ON_2                  2
#define INTERRUPT_ALWAYS                3

#define NO_PING                         0
#define PING_PKT                        1

#define STATUS_CTRL_NONE                0
#define STATUS_CTRL_WRITE               1

#define Z_LAST                          0
#define Z_MORE                          1

typedef struct Async_OUTPUT_MORE_descriptor_Struct
{
    QUADLET reqCount        :16;
    QUADLET rsvd1           :2;
    QUADLET b               :2;     // 0 (BRANCH_NEVER)
    QUADLET rsvd2           :4;
    QUADLET key             :3;     // 0 (KEY_ADDRESS)
    QUADLET rsvd3           :1;
    QUADLET cmd             :4;     // 0 (OUTPUT_MORE)
    QUADLET dataAddress;
    QUADLET rsvd4;
    QUADLET rsvd5;
} Async_OUTPUT_MORE_descriptor, *PAsync_OUTPUT_MORE_descriptor;

typedef struct Async_OUTPUT_MORE_Immediate_descriptor_Struct
{
    QUADLET reqCount        :16;    // 8 or 16
    QUADLET rsvd1           :2;
    QUADLET b_zeros         :2;     // 0 (BRANCH_NEVER)
    QUADLET rsvd2           :4;
    QUADLET key             :3;     // 2 (KEY_IMMEDIATE)
    QUADLET rsvd3           :1;
    QUADLET cmd             :4;     // 0 (OUTPUT_MORE)
    QUADLET rsvd4;
    QUADLET rsvd5;
    QUADLET timeStamp       :16;
    QUADLET rsvd6           :16;
    QUADLET first_quadlet;
    QUADLET second_quadlet;
    QUADLET third_quadlet;
    QUADLET forth_quadlet;
} Async_OUTPUT_MORE_Immediate_descriptor, *PAsync_OUTPUT_MORE_Immediate_descriptor;

typedef struct Async_OUTPUT_LAST_descriptor_Struct
{
    QUADLET reqCount        :16;
    QUADLET rsvd1           :2;
    QUADLET b               :2;     // 3 (BRANCH_ALWAYS)
    QUADLET i               :2;     // 3 (INTERRUPT_ALWAYS), 1 (INTERRUPT_NO_ACK), or 0 (INTERRUPT_NEVER)
    QUADLET rsvd2           :1;
    QUADLET p               :1;     // 0 (NO_PING), 1 (PING_PKT)
    QUADLET key             :3;     // 0 (KEY_ADDRESS)
    QUADLET rsvd3           :1;
    QUADLET cmd             :4;     // 1 (OUTPUT_LAST)
    QUADLET dataAddress;
    QUADLET Z               :4;     // 0 (Z_LAST), 1 (Z_MORE)
    QUADLET branchAddress   :28;
    QUADLET timeStamp       :16;
    QUADLET xferStatus      :16;
} Async_OUTPUT_LAST_descriptor, *PAsync_OUTPUT_LAST_descriptor;

typedef struct Async_OUTPUT_LAST_Immediate_descriptor_Struct
{
    QUADLET reqCount        :16;    // 8, 12 or 16
    QUADLET rsvd1           :2;
    QUADLET b               :2;     // 3 (BRANCH_ALWAYS)
    QUADLET i               :2;     // 3 (INTERRUPT_ALWAYS), 1 (INTERRUPT_NO_ACK), 0 (INTERUPT_NEVER)
    QUADLET rsvd2           :1;
    QUADLET p               :1;     // 0 (NO_PING), 1 (PING_PKT)
    QUADLET key             :3;     // 2 (KEY_IMMEDIATE)
    QUADLET rsrd3           :1;
    QUADLET cmd             :4;     // 1 (OUTPUT_LAST)
    QUADLET rsvd4;
    QUADLET Z               :4;
    QUADLET branchAddress   :28;
    QUADLET timeStamp       :16;
    QUADLET xferStatus      :16;
    QUADLET first_quadlet;
    QUADLET second_quadlet;
    QUADLET third_quadlet;
    QUADLET fourth_quadlet;
} Async_OUTPUT_LAST_Immediate_descriptor, *PAsync_OUTPUT_LAST_Immediate_descriptor;

typedef struct Async_INPUT_MORE_descriptor_Struct
{
    QUADLET reqCount        :16;
    QUADLET rsvd1           :2;
    QUADLET b               :2;     // 3 (BRANCH_ALWAYS)
    QUADLET i               :2;     // 3 (INTERRUPT_ALWAYS), 0 (INTERRUPT_NEVER)
    QUADLET rsvd2           :2;
    QUADLET key             :3;     // 0 (KEY_ADDRESS)
    QUADLET s               :1;     // 1 (STATUS_CTRL_WRITE)
    QUADLET cmd             :4;     // 2 (INPUT_MORE)
    QUADLET rsvd3           :2;     // 0
    QUADLET dataAddress     :30;
    QUADLET Z               :4;     // 0 (Z_LAST), 1 (Z_MORE)
    QUADLET branchAddress   :28;
    QUADLET resCount        :16;
    QUADLET xferStatus      :16;
} Async_INPUT_MORE_descriptor, *PAsync_INPUT_MORE_descriptor;

typedef struct Isoc_INPUT_MORE_descriptor_Struct
{
    QUADLET reqCount        :16;
    QUADLET w               :2;
    QUADLET b               :2;     // 0 (BRANCH_ALWAYS)
    QUADLET rsvd1           :2;
    QUADLET i               :2;
    QUADLET key             :3;     // 0 (KEY_ADDRESS)
    QUADLET s               :1;
    QUADLET cmd             :4;     // 2 (INPUT_MORE)
    QUADLET dataAddress;
    QUADLET Z               :4;
    QUADLET branchAddress   :28;
    QUADLET resCount        :16;
    QUADLET xferStatus      :16;
} Isoc_INPUT_MORE_descriptor, *PIsoc_INPUT_MORE_descriptor;

typedef struct STORE_VALUE_descriptor_Struct
{
    QUADLET storeDoublet    :16;
    QUADLET rsvd1           :8;
    QUADLET key             :3;     // 6 (KEY_SYSTEM_MEMORY)
    QUADLET cmd             :4;     // 0 (STORE_VALUE)
    QUADLET rsvd2           :2;
    QUADLET dataAddress     :30;
    QUADLET Z               :4;
    QUADLET skipAddress     :28;
    QUADLET rsvd3;
} STORE_VALUE_descriptor, *PSTORE_VALUE_descriptor;

// DMA Data Packet Structures and Definitions

#define tCode_WRITE_QUADLET_REQUEST 0
#define tCode_WRITE_BLOCK_REQUEST   1
#define tCode_WRITE_RESPONSE        2
#define tCode_RESERVED_3            3
#define tCode_READ_QUADLET_REQUEST  4
#define tCode_READ_BLOCK_REQUEST    5
#define tCode_READ_QUADLET_RESPONSE 6
#define tCode_READ_BLOCK_RESPONSE   7
#define tCode_CYCLE_START           8
#define tCode_LOCK_REQUEST          9
#define tCode_ISOCH_DATA_BLOCK      10
#define tCode_LOCK_RESPONSE         11
#define tCode_RESERVED_12           12
#define tCode_RESERVED_13           13
#define tCode_RESERVED_14           14
#define tCode_RESERVED_15           15

#define retry_1                     0
#define retry_X                     1
#define retry_A                     2
#define retry_B                     3

#define MAX_TLABEL                  0x3F

#define spd_100                     0
#define spd_200                     1
#define spd_400                     2
#define spd_RESERVED_3              3
#define spd_RESERVED_4              4
#define spd_RESERVED_5              5
#define spd_RESERVED_6              6
#define spd_RESERVED_7              7

#define srcBusID_DEFAULT            0

typedef struct Quadlet_read_request_transmit_Sturct
{
    QUADLET rsvd1                   :4;
    QUADLET tCode                   :4;     // 4 = Quadlet_read_request_transmit
    QUADLET rt                      :2;
    QUADLET tLabel                  :6;
    QUADLET spd                     :3;
    QUADLET rsvd2                   :4;
    QUADLET srcBusID                :1;
    QUADLET rsvd3                   :8;
    QUADLET destinationOffsetHigh   :16;
    QUADLET destinationID           :16;
    QUADLET destinationOffsetLow;
} Quadlet_read_request_transmit, *PQuadlet_read_request_transmit;

typedef struct Quadlet_write_request_transmit_Sturct
{
    QUADLET rsvd1                   :4;
    QUADLET tCode                   :4;
    QUADLET rt                      :2;
    QUADLET tLabel                  :6;
    QUADLET spd                     :3;
    QUADLET rsvd2                   :4;
    QUADLET srcBusID                :1;
    QUADLET rsvd3                   :8;
    QUADLET destinationOffsetHigh   :16;
    QUADLET destinationID           :16;
    QUADLET destinationOffsetLow;
    QUADLET quadletData;
} Quadlet_write_request_transmit, *PQuadlet_write_request_transmit;

typedef struct Block_read_request_transmit_Sturct
{
    QUADLET rsvd1                   :4;
    QUADLET tCode                   :4;
    QUADLET rt                      :2;
    QUADLET tLabel                  :6;
    QUADLET spd                     :3;
    QUADLET rsvd2                   :4;
    QUADLET srcBusID                :1;
    QUADLET rsvd3                   :8;
    QUADLET destinationOffsetHigh   :16;
    QUADLET destinationID           :16;
    QUADLET destinationOffsetLow;
    QUADLET rsvd4                   :16;
    QUADLET dataLength              :16;
} Block_read_request_transmit, *PBlock_read_request_transmit;

typedef struct Block_write_request_transmit_Sturct
{
    QUADLET rsvd1                   :4;
    QUADLET tCode                   :4;
    QUADLET rt                      :2;
    QUADLET tLabel                  :6;
    QUADLET spd                     :3;
    QUADLET rsvd2                   :4;
    QUADLET srcBusID                :1;
    QUADLET rsvd3                   :8;
    QUADLET destinationOffsetHigh   :16;
    QUADLET destinationID           :16;
    QUADLET destinationOffsetLow;
    QUADLET blockData[1];
} Block_write_request_transmit, *PBlock_write_request_transmit;

typedef struct Quadlet_read_request_receive_Struct
{
    QUADLET rsvd1                   :4;
    QUADLET tCode                   :4;
    QUADLET rt                      :2;
    QUADLET tLabel                  :6;
    QUADLET destinationID           :16;
    QUADLET destinationOffsetHigh   :16;
    QUADLET sourceID                :16;
    QUADLET destinationOffsetLow;
    QUADLET timeStamp               :16;
    QUADLET xferStatus              :16;
} Quadlet_read_request_receive, *PQuadlet_read_request_receive;

typedef struct Quadlet_write_request_receive_Struct
{
    QUADLET rsvd1                   :4;
    QUADLET tCode                   :4;
    QUADLET rt                      :2;
    QUADLET tLabel                  :6;
    QUADLET destinationID           :16;
    QUADLET destinationOffsetHigh   :16;
    QUADLET sourceID                :16;
    QUADLET destinationOffsetLow;
    QUADLET quadlet_data;
    QUADLET timeStamp               :16;
    QUADLET xferStatus              :16;
} Quadlet_write_request_receive, *PQuadlet_write_request_receive;

typedef struct Block_read_request_receive_Struct
{
    QUADLET rsvd1                   :4;
    QUADLET tCode                   :4;
    QUADLET rt                      :2;
    QUADLET tLabel                  :6;
    QUADLET destinationID           :16;
    QUADLET destinationOffsetHigh   :16;
    QUADLET sourceID                :16;
    QUADLET destinationOffsetLow;
    QUADLET rsvd2                   :16;
    QUADLET dataLength              :16;
    QUADLET timeStamp               :16;
    QUADLET xferStatus              :16;
} Block_read_request_receive, *PBlock_read_request_receive;

typedef struct Block_write_request_receive_Struct
{
    QUADLET rsvd1                   :4;
    QUADLET tCode                   :4;
    QUADLET rt                      :2;
    QUADLET tLabel                  :6;
    QUADLET destinationID           :16;
    QUADLET destinationOffsetHigh   :16;
    QUADLET sourceID                :16;
    QUADLET destinationOffsetLow;
    QUADLET rsvd2                   :16;
    QUADLET dataLength              :16;
    QUADLET block_data[1];                  // First quadlet of block
    // Real offset = offsetof(timeStamp) + dataLength - (dataLength % sizeof(QUADLET))
    QUADLET timeStamp               :16;    
    // Real offset = offsetof(xferStatus) + dataLength - (dataLength % sizeof(QUADLET))
    QUADLET xferStatus              :16;
} Block_write_request_receive, *PBlock_write_request_receive;

typedef struct Write_response_receive_Struct
{
    QUADLET rsvd1                   :4;
    QUADLET tCode                   :4;
    QUADLET rt                      :2;
    QUADLET tLabel                  :6;
    QUADLET destinationID           :16;
    QUADLET rsvd2                   :12;
    QUADLET rCode                   :4;
    QUADLET sourceID                :16;
    QUADLET rsvd3;
    QUADLET timeStamp               :16;
    QUADLET xferStatus              :16;
} Write_response_receive, *PWrite_response_receive;

typedef struct Quadlet_read_response_receive_Struct
{
    QUADLET rsvd1                   :4;
    QUADLET tCode                   :4;
    QUADLET rt                      :2;
    QUADLET tLabel                  :6;
    QUADLET destinationID           :16;
    QUADLET rsvd2                   :12;
    QUADLET rCode                   :4;
    QUADLET sourceID                :16;
    QUADLET rsvd3;
    QUADLET quadlet_data;
    QUADLET timeStamp               :16;
    QUADLET xferStatus              :16;
} Quadlet_read_response_receive, *PQuadlet_read_response_receive;

typedef struct Block_read_response_receive_Struct
{
    QUADLET rsvd1                   :4;
    QUADLET tCode                   :4;
    QUADLET rt                      :2;
    QUADLET tLabel                  :6;
    QUADLET destinationID           :16;
    QUADLET rsvd2                   :12;
    QUADLET rCode                   :4;
    QUADLET sourceID                :16;
    QUADLET rsvd3;
    QUADLET rsvd4                   :16;
    QUADLET dataLength              :16;
    QUADLET block_data[1];                  // First quadlet of data block
    // Real offset = offsetof(timeStamp) + dataLength - (dataLength % sizeof(QUADLET))
    QUADLET timeStamp               :16;
    // Real offset = offsetof(xferStatus) + dataLength - (dataLength % sizeof(QUADLET))
    QUADLET xferStatus              :16;
} Block_read_response_receive, *PBlock_read_response_receive;

// Definitions for DUMP ROM

typedef struct ADDRESS_OFFSET_Struct
{
    __u16   Off_High;
    __u32   Off_Low;
} ADDRESS_OFFSET, *PADDRESS_OFFSET;

// Misc definitions
typedef union _LARGE_UINT
{     
    struct
    {        
        __u32 LowPart; 
        __u32 HighPart;
    }fld;
//    ULONGLONG QuadPart;
    __u64 QuadPart;
} LARGE_UINT; 

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// IEEE1212 and IEEE1394 Definitions
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

typedef struct _1394BusInfoBlockStruct          
{                                                           
    // First quadlet
    QUADLET     bus_name;                   // Always 0x31333934 or "1394"
                                                             
    // Second quadlet                                    
    QUADLET     reserved1:                  12;
    QUADLET     max_rec:                    4;
    QUADLET     cyc_clk_acc:                8;
    QUADLET     reserved2:                  4;
    QUADLET     bmc:                        1;
    QUADLET     isc:                        1;
    QUADLET     cmc:                        1;
    QUADLET     irmc:                       1;
                                                        
    // Third quadlet                                
    QUADLET     chip_id_hi:                 8;
    QUADLET     node_vendor_id:             24;

    // Fourth quadlet
    QUADLET     chip_id_lo;

} _1394BusInfoBlockT, *P_1394BusInfoBlockT;

#define BUS_NAME_1394   0x31333934              
                                                            
typedef struct CfgRomHeaderStruct
{
    union
    {
        // General 1394 CFG ROM format
        struct
        {
            QUADLET     rom_crc_value:      16;
            QUADLET     crc_length:         8;
            QUADLET     info_length:        8;      // Should be 0x04 (for 1394)
            _1394BusInfoBlockT  bus_info_block;
        }gcr;

        // Minimal CFG ROM format
        struct
        {
            QUADLET     vendor_id:          24;
            QUADLET     info_length:        8;      // Should be 0x01
        }mcr;
    }u;
} CfgRomHeaderT, *PCfgRomHeaderT;

typedef struct BlockHeaderStruct
{
    QUADLET crc:                            16;
    QUADLET length:                         16;
} BlockHeaderT, *PBlockHeaderT;

typedef struct DirEntryStruct
{
    union
    {
        // Immediate value (key_type = 00b)
        struct
        {
            QUADLET value:                  24;
            QUADLET key_value:              6;
            QUADLET key_type:               2;
        }iv;

        // Pointer to location relative to XXXXFFFF:F0000000 (key_type = 01b)
        struct
        {
            QUADLET csr_offset:             24;
            QUADLET key_value:              6;
            QUADLET key_type:               2;
        }co;

        // Pointer to location relative to the position of this value (key_type = 10b & 11b)
        struct
        {
            QUADLET indirect_offset:        24;
            QUADLET key_value:              6;
            QUADLET key_type:               2;
        }io;

        // General format
        struct
        {
            QUADLET value:                  24;
            QUADLET key:                    8;
        }general;

        // QUADLET format
        QUADLET     q;

    }u;
} DirEntryT, *PDirEntryT;

// Definitions for values of key_type
// NOTE:  1212 spec does not specify the exact name of these values
// so we'll name them
#define KT_IMMEDIATE_VALUE                  0
#define KT_CSR_OFFSET                       1
#define KT_LEAF_INDIRECT_OFFSET             2
#define KT_DIRECTORY_INDIRECT_OFFSET        3

// Definitions for values of key_value
// NOTE:  We will name these exactly as the 1212 spec defines
#define KV_UNKNOWN                          0x00
#define Textual_Descriptor                  0x01
#define Bus_Dependent_Info                  0x02
#define Module_Vendor_Id                    0x03
#define Module_Hw_Version                   0x04
#define Module_Spec_Id                      0x05
#define Module_Sw_Version                   0x06
#define Module_Dependent_Info               0x07
#define Node_Vendor_Info                    0x08
#define Node_Hw_Version                     0x09
#define Node_Spec_Id                        0x0A
#define Node_Sw_Version                     0x0B
#define Node_Capabilities                   0x0C
#define Node_Unique_Id                      0x0D
#define Node_Units_Extent                   0x0E
#define Node_Memory_Extent                  0x0F
#define Node_Dependent_Info                 0x10
#define Unit_Directory                      0x11
#define Unit_Spec_Id                        0x12
#define Unit_Sw_Version                     0x13
#define Unit_Dependent_Info                 0x14
#define Unit_Location                       0x15
#define Unit_Poll_Mask                      0x16
// 1394 specific key_value definitions
#define Unit_Power_Requirements             0x30
// SBP-2 specific key_value definitions
#define Model_Id                            0x17
#define Command_Set_Spec_Id                 0x38
#define Command_Set                         0x39
#define Unit_Characteristics                0x3A
#define Command_Set_Revision                0x3B
#define Firmware_Revision                   0x3C
#define Management_Agent                    0x14    // 0x54 (key_type = KT_CSR_OFFSET)
#define Logical_Unit_Directory              0x14    // 0xD4 (key_type = KT_DIRECTORY_INDIRECT_OFFSET)
#define Logical_Unit_Number                 0x14

// Misc SBP-2 Definitions
#define SBP2_Unit_Spec_Id                   0x00609E
#define SBP2_Unit_Sw_Version                0x010483
#define SBP2_Command_Set_Spec_Id            0x00609E
#define SBP2_Command_Set                    0x0104D8

typedef struct NodeCapabiltiesStruct
{
    QUADLET init:                           1;
    QUADLET ded:                            1;
    QUADLET off:                            1;
    QUADLET atn:                            1;
    QUADLET elo:                            1;
    QUADLET Reserved1:                      1;
    QUADLET drg:                            1;
    QUADLET lst:                            1;
    QUADLET fix:                            1;
    QUADLET _64:                            1;
    QUADLET prv:                            1;
    QUADLET bas:                            1;
    QUADLET ext:                            1;
    QUADLET _int:                           1;
    QUADLET ms:                             1;
    QUADLET spt:                            1;
    QUADLET Reserved2:                      8;
} NodeCapabilitiesT, *PNodeCapabilitiesT;

typedef struct NodeUnitsExtentStruct
{
    QUADLET number:                         16;
    QUADLET align:                          6;
    QUADLET Reserved:                       2;
} NodeUnitsExtentT, *PNodeUnitsExtentT;

typedef NodeUnitsExtentT NodeMemoryExtentT, *PNodeMemoryExtentT;

typedef struct UnitPollMaskStruct
{
    QUADLET poll_mask:                      16;
    QUADLET Reserved:                       8;
} UnitPollMaskT, *PUnitPollMaskT;

typedef struct LogicalUnitNumberStruct
{
    QUADLET lun:                            16;
    QUADLET device_type:                    5;
    QUADLET Reserved1:                      1;
    QUADLET o:                              1;
    QUADLET Reserved2:                      1;
} LogicalUnitNumberT, *PLogicalUnitNumberT;

// global variables

// define EXTERN if it isn't to extern
#ifndef EXTERN
#define EXTERN      extern
#endif

#ifdef _APP_MAIN_MODULE_

// SYM13FW500 Core 1394 Addresses
ADDRESS_OFFSET STATE_CLEAR_ADDR             =   {0xFFFF, 0xF0000000};
ADDRESS_OFFSET STATE_SET_ADDR               =   {0xFFFF, 0xF0000004};
ADDRESS_OFFSET NODE_IDS_ADDR                =   {0xFFFF, 0xF0000008};
ADDRESS_OFFSET RESET_START_ADDR             =   {0xFFFF, 0xF000000C};
ADDRESS_OFFSET SPLIT_TIMEOUT_H_ADDR         =   {0xFFFF, 0xF0000018};
ADDRESS_OFFSET SPLIT_TIMEOUT_L_ADDR         =   {0xFFFF, 0xF000001C};
ADDRESS_OFFSET BUSY_TIMEOUT_ADDR            =   {0xFFFF, 0xF0000210};
ADDRESS_OFFSET CONFIG_ROM_ADDR              =   {0xFFFF, 0xF0000400};
ADDRESS_OFFSET FIRMWARE_READ_ADDR           =   {0x0000, 0x00004000};

// SBP-2 CSR Addresses
ADDRESS_OFFSET CSR_BASE_ADDR                =   {0xFFFF, 0xF0000000};
ADDRESS_OFFSET CSR_ADDRESS_ADDR             =   {0xFFFF, 0xF0000400};
ADDRESS_OFFSET FETCH0_AGENT_ADDR            =   {0xFFFF, 0xF0010000};
ADDRESS_OFFSET FETCH0_AGENT_RESET_ADDR      =   {0xFFFF, 0xF0010004};
ADDRESS_OFFSET FETCH0_AGENT_ORB_ADDR        =   {0xFFFF, 0xF0010008};
ADDRESS_OFFSET FETCH1_AGENT_ADDR            =   {0xFFFF, 0xF0020000};
ADDRESS_OFFSET FETCH1_AGENT_RESET_ADDR      =   {0xFFFF, 0xF0020004};
ADDRESS_OFFSET FETCH1_AGENT_ORB_ADDR        =   {0xFFFF, 0xF0020008};
ADDRESS_OFFSET MAN_AGENT_ADDR               =   {0xFFFF, 0xF0030000};

#else

extern ADDRESS_OFFSET STATE_CLEAR_ADDR;
extern ADDRESS_OFFSET STATE_SET_ADDR;
extern ADDRESS_OFFSET NODE_IDS_ADDR;
extern ADDRESS_OFFSET RESET_START_ADDR;
extern ADDRESS_OFFSET SPLIT_TIMEOUT_H_ADDR;
extern ADDRESS_OFFSET SPLIT_TIMEOUT_L_ADDR;
extern ADDRESS_OFFSET BUSY_TIMEOUT_ADDR;
extern ADDRESS_OFFSET CONFIG_ROM_ADDR;
extern ADDRESS_OFFSET CSR_BASE_ADDR;
extern ADDRESS_OFFSET CSR_ADDRESS_ADDR;
extern ADDRESS_OFFSET FETCH0_AGENT_ADDR;
extern ADDRESS_OFFSET FETCH0_AGENT_RESET_ADDR;
extern ADDRESS_OFFSET FETCH0_AGENT_ORB_ADDR;
extern ADDRESS_OFFSET FETCH1_AGENT_ADDR;
extern ADDRESS_OFFSET FETCH1_AGENT_RESET_ADDR;
extern ADDRESS_OFFSET FETCH1_AGENT_ORB_ADDR;
extern ADDRESS_OFFSET MAN_AGENT_ADDR;

#endif



//#define   XXX 0x000
//typedef union
//{
//  QUADLET reg;
//  struct
//  {
//      __u32 Rsvd :16;
//      __u32 Rsvd1 :7;
//  }fld;
//}XXX_REG;


#endif